// Copyright (c) 2019-2022 Dr. Colin Hirsch and Daniel Frey
// Please see LICENSE for license or visit https://github.com/taocpp/PEGTL/

#ifndef PXR_PEGTL_REQUIRE_APPLY_HPP
#define PXR_PEGTL_REQUIRE_APPLY_HPP

#include "config.hpp"

namespace PXR_PEGTL_NAMESPACE
{
   struct require_apply
   {};

}  // namespace PXR_PEGTL_NAMESPACE

#endif
