// Copyright (c) 2017-2022 Dr. Colin Hirsch and Daniel Frey
// Please see LICENSE for license or visit https://github.com/taocpp/PEGTL/

#ifndef PXR_PEGTL_CONFIG_HPP
#define PXR_PEGTL_CONFIG_HPP

#define PXR_PEGTL_NAMESPACE pxr_pegtl

#endif
