// Copyright (c) 2020-2022 Dr. Colin Hirsch and Daniel Frey
// Please see LICENSE for license or visit https://github.com/taocpp/PEGTL/

#ifndef PXR_PEGTL_CONTRIB_FORWARD_HPP
#define PXR_PEGTL_CONTRIB_FORWARD_HPP

#include "../config.hpp"

namespace PXR_PEGTL_NAMESPACE
{
   template< typename Name, typename Rule, typename = void >
   struct analyze_traits;

}  // namespace PXR_PEGTL_NAMESPACE

#endif
